<?php
/**
 * Administration panel - Rights by item editor
 *
 * @package Cotonti
 * @copyright (c) Cotonti Team
 * @license https://github.com/Cotonti/Cotonti/blob/master/License.txt
 */

(defined('COT_CODE') && defined('COT_ADMIN')) or die('Wrong URL.');

list(cot::$usr['auth_read'], cot::$usr['auth_write'], cot::$usr['isadmin']) = cot_auth('users', 'a');
cot::$usr['isadmin'] &= cot_auth('admin', 'a', 'A');
if (cot::$usr['maingrp'] == COT_GROUP_SUPERADMINS) {
    cot::$usr['auth_read'] = true;
    cot::$usr['auth_write'] = true;
    cot::$usr['isadmin'] = true;
}
cot_block(cot::$usr['isadmin']);

$t = new XTemplate(cot_tplfile('admin.rightsbyitem', 'core'));

$ic = cot_import('ic', 'G', 'ALP');
$io = cot_import('io', 'G', 'ALP');
$advanced = cot_import('advanced', 'G', 'BOL');

cot::$L['adm_code']['admin'] = cot::$L['Administration'];
cot::$L['adm_code']['message'] = cot::$L['Messages'];

/* === Hook === */
foreach (cot_getextplugins('admin.rightsbyitem.first') as $pl) {
	include $pl;
}
/* ===== */

if ($a == 'update') {
	$mask = array();
	$auth = cot_import('auth', 'P', 'ARR');
    $items = cot_import('items', 'P', 'TXT');
    if (!empty($items)) {
        // Update rights for these user groups only
        $items = json_decode($items, true);
        $items = !empty($items) ? $items : [];
    } else {
        // Update rights for all groups
        $sql = cot::$db->query(
            'SELECT a.* FROM ' . cot::$db->auth . ' as a ' .
            'LEFT JOIN ' . cot::$db->groups . ' AS g ON g.grp_id=a.auth_groupid ' .
            "WHERE auth_code = ? AND auth_option = ? AND grp_skiprights = 0 ORDER BY grp_level DESC, grp_id DESC",
            [$ic, $io]
        );
        $items = [];
        while ($row = $sql->fetch()) {
            $items[$row['auth_groupid']] = $row['auth_rights'];
        }
    }

	/* === Hook === */
	foreach (cot_getextplugins('admin.rightsbyitem.update') as $pl) {
		include $pl;
	}
	/* ===== */

	//foreach ($auth as $i => $j) {
    foreach ($items as $groupId => $oldMaskValue) {
        $newValue = !empty($auth[$groupId]) && is_array($auth[$groupId]) ? $auth[$groupId] : [];

        $mask = 0;
        foreach ($newValue as $l => $m) {
            $mask += cot_auth_getvalue($l);
        }

        $oldRights = cot::$db->query("SELECT auth_id, auth_rights FROM ".cot::$db->auth.
            " WHERE auth_groupid=? AND auth_code=? AND auth_option=?", [$groupId, $ic, $io])->fetch();

        if (!empty($oldRights) && $oldRights['auth_rights'] != $mask) {
            cot::$db->update(cot::$db->auth,  array('auth_rights' => $mask, 'auth_setbyuserid' => cot::$usr['id']),
                "auth_id=".$oldRights['auth_id']);
        }
	}

	cot_auth_reorder();
	cot_auth_clear('all');

	cot_message('Updated');

    $urlParams = ['m' => 'rightsbyitem', 'ic' => $ic, 'io' => $io,];
    if ($advanced) {
        $urlParams['advanced'] = 1;
    }
    cot_redirect(cot_url('admin', $urlParams, '', true));
}

$sql = cot::$db->query('SELECT a.*, u.user_name, g.grp_name, g.grp_level FROM ' . cot::$db->auth . ' as a ' .
'LEFT JOIN ' . cot::$db->users . ' AS u ON u.user_id=a.auth_setbyuserid '.
'LEFT JOIN ' . cot::$db->groups . ' AS g ON g.grp_id=a.auth_groupid ' .
"WHERE auth_code = ? AND auth_option = ? AND grp_skiprights = 0 ORDER BY grp_level DESC, grp_id DESC", [$ic, $io]);

cot_die($sql->rowCount() == 0);

$title = '';

/* === Hook for the plugins === */
foreach (cot_getextplugins('admin.rightsbyitem.case') as $pl) {
	include $pl;
}
/* ===== */

if ($ic == 'message' || $ic == 'admin') {
	$adminpath[] = array(cot_url('admin'), cot::$L['adm_code'][$ic]);

} else {
	$adminpath[] = array(cot_url('admin', 'm=extensions'), cot::$L['Extensions']);
	if ($ic == 'plug') {
        $itemTitle = !empty($cot_plugins_enabled[$io]) ? $cot_plugins_enabled[$io]['title'] : $io;
        $title = $itemTitle;
		$adminpath[] = array(cot_url('admin', 'm=extensions&a=details&pl='.$io), $itemTitle);

	} elseif ($ic == 'structure') {
		$adminpath[] = array(cot_url('admin', 'm=structure'), cot::$L['Structure']);

	} else {
        $itemTitle = !empty($cot_modules[$ic]) ? $cot_modules[$ic]['title'] : $ic;
        $title = $itemTitle;
        $adminpath[] = array(cot_url('admin', 'm=extensions&a=details&mod='.$ic), $itemTitle);
		if ($io != 'a') {
			$adminpath[] = array(cot_url('admin', 'm=structure&n='.$ic), cot::$L['Structure']);
            $itemTitle = !empty(cot::$structure[$ic][$io]) ? cot::$structure[$ic][$io]['title'] : $io;
            $title .= '(' . $itemTitle . ')';
            $adminpath[] = array(cot_url('admin', 'm=structure&n='.$ic.'&al='.$io), $itemTitle);
		}
	}
}

//m=extensions&a=details&mod=page
$adminpath[] = array(cot_url('admin', 'm=rightsbyitem&ic='.$ic.'&io='.$io), cot::$L['Rights']);
($advanced) && $adminpath[] = array(cot_url('admin', 'm=rightsbyitem&ic='.$ic.'&io='.$io.'&advanced=1'), cot::$L['More']);
$adminTitle = cot::$L['Rights'];

$adv_columns = ($advanced) ? 8 : 3;
$adv_columns = (!$advanced && $ic == 'page') ? 4 : $adv_columns;

$l_custom1 = ($ic == 'page') ? cot::$L['Download'] : cot::$L['Custom'].' #1';


// All items that are present in rights edit form
$items = [];
while ($row = $sql->fetch()) {
	$link = cot_url('admin', 'm=rights&g=' . $row['auth_groupid']);
	cot_rights_parseline($row, $row['grp_name'], $link);
    $items[$row['auth_groupid']] = $row['auth_rights'];
}
$sql->closeCursor();

$is_adminwarnings = isset($adminwarnings);

$urlParams = ['m' => 'rightsbyitem', 'ic' => $ic, 'io' => $io, 'a' => 'update',];
if ($advanced) {
    $urlParams['advanced'] = 1;
}

$pageTitle = $adminTitle;
if (!empty($title)) {
    $pageTitle .= ': ' . $title;
}
$t->assign(array(
    'ADMIN_RIGHTSBYITEM_TITLE' => $pageTitle,
	'ADMIN_RIGHTSBYITEM_FORM_URL' => cot_url('admin', $urlParams),
    'ADMIN_RIGHTSBYITEM_FORM_ITEMS' => '', // Update rights for all groups
	'ADMIN_RIGHTSBYITEM_ADVANCED_URL' => cot_url('admin', 'm=rightsbyitem&ic=' . $ic . '&io=' . $io .
        '&advanced=1'),
	'ADMIN_RIGHTSBYITEM_ADV_COLUMNS' => $adv_columns,
	'ADMIN_RIGHTSBYITEM_4ADV_COLUMNS' => 4 + $adv_columns
));

cot_display_messages($t);

/* === Hook === */
foreach (cot_getextplugins('admin.rightsbyitem.tags') as $pl) {
	include $pl;
}
/* ===== */

$t->parse('MAIN');
$adminmain = $t->text('MAIN');

$t->parse('RIGHTSBYITEM_HELP');
$adminhelp = $t->text('RIGHTSBYITEM_HELP');

function cot_rights_parseline($row, $title, $link)
{
	global $L, $advanced, $t, $out, $ic;

	$mn['R'] = 1;
	$mn['W'] = 2;

	if ($advanced || $ic == 'page') {
		$mn['1'] = 4;
	} else {
		$rv['1'] = 4;
	}

	if ($advanced) {
		$mn['2'] = 8;
		$mn['3'] = 16;
		$mn['4'] = 32;
		$mn['5'] = 64;

    } else {
		$rv['2'] = 8;
		$rv['3'] = 16;
		$rv['4'] = 32;
		$rv['5'] = 64;
	}
	$mn['A'] = 128;

	foreach ($mn as $code => $value) {
		$state[$code] = (($row['auth_rights'] & $value) == $value) ? true : false;
		$locked[$code] = (($row['auth_rights_lock'] & $value) == $value) ? true : false;

        // Deprecated. Use tags instead
        $out['tpl_rights_parseline_locked'] = $locked[$code];
		$out['tpl_rights_parseline_state'] = $state[$code];

		$t->assign(array(
			'ADMIN_RIGHTSBYITEM_ROW_ITEMS_NAME' => 'auth['.$row['auth_groupid'].']['.$code.']',
			'ADMIN_RIGHTSBYITEM_ROW_ITEMS_CHECKED' => ($state[$code]) ? " checked=\"checked\"" : '',
			'ADMIN_RIGHTSBYITEM_ROW_ITEMS_DISABLED' => ($locked[$code]) ? " disabled=\"disabled\"" : '',
            'ADMIN_RIGHTSBYITEM_ROW_ITEMS_LOCKED' => $locked[$code],
            'ADMIN_RIGHTSBYITEM_ROW_ITEMS_STATE' => $state[$code],
		));
		$t->parse('MAIN.RIGHTSBYITEM_ROW.ROW_ITEMS');
	}

	if (!$advanced) {
		$preserve = '';
		foreach ($rv as $code => $value) {
			if (($row['auth_rights'] & $value) == $value) {
				$preserve .= '<input type="hidden" name="auth['.$row['auth_groupid'].']['.$code.']" value="1" />';
			}
		}
		$t->assign('ADMIN_RIGHTSBYITEM_ROW_PRESERVE', $preserve);
	}

	$t->assign(array(
		'ADMIN_RIGHTSBYITEM_ROW_TITLE' => $title,
		'ADMIN_RIGHTSBYITEM_ROW_LINK' => $link,
		'ADMIN_RIGHTSBYITEM_ROW_USER' => cot_build_user($row['auth_setbyuserid'], $row['user_name']),
		'ADMIN_RIGHTSBYITEM_ROW_JUMPTO' => cot_url('users', 'g='.$row['auth_groupid']),
	));
	$t->parse('MAIN.RIGHTSBYITEM_ROW');
}
